from typing import Any
from django.http import HttpRequest
from django.http.response import HttpResponse as HttpResponse
from django.shortcuts import get_object_or_404, redirect, render
from django.urls import reverse, reverse_lazy
from django.views import generic
from django.contrib.auth.mixins import LoginRequiredMixin, PermissionRequiredMixin, UserPassesTestMixin
from django.core.exceptions import PermissionDenied

from webapp.forms import ArticleForm, CommentForm
from webapp.models import Article


class ArticleDetailView(PermissionRequiredMixin, generic.DetailView):
    pk_url_kwarg = 'id'
    model = Article
    template_name = 'articles/detail.html'
    permission_required = 'webapp.can_eat_pizza'
    
    def get_context_data(self, **kwargs):
        comments = self.object.comments.order_by('-created_at')
        return super().get_context_data(
            **kwargs, 
            comments=comments,
            comments_form=CommentForm,
        )


class ArticleCreateView(PermissionRequiredMixin, generic.CreateView):
    model = Article
    template_name = 'articles/article.html'
    form_class = ArticleForm
    permission_required = 'webapp.add_article'
    permission_denied_message = 'У вас нет прав'
    
    def form_valid(self, form):
        form.instance.author = self.request.user
        return super().form_valid(form)

    def get_success_url(self):
        return reverse('articles_detail', kwargs={'id': self.object.id})


class ArticleUpdateView(PermissionRequiredMixin, generic.UpdateView):
    template_name = 'articles/article.html'
    model = Article
    form_class = ArticleForm
    context_object_name = 'article'
    pk_url_kwarg = 'id'
    extra_context = {'btn_text': 'Update'}
    permission_required = 'webapp.change_article'

    def has_permission(self) -> bool:
        return super().has_permission() or self.get_object().author == self.request.user

    def get_success_url(self):
        return reverse('articles_detail', kwargs={'id': self.object.id})


class DeleteView(generic.View):
    template_name = None
    model = None
    context_object_name = None
    pk_url_kwarg = None
    success_url = None
    
    def dispatch(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        self.object = self.get_object()
        return super().dispatch(request, *args, **kwargs)

    def get(self, request, *args, **kwargs):
        return self.render_to_response()

    def post(self, request, *args, **kwargs):
        self.perform_delete()
        return redirect(self.get_success_url())

    def perform_delete(self):
        self.object.delete()

    def get_object(self):
        return get_object_or_404(self.model, id=self.kwargs.get(self.pk_url_kwarg))
    
    def get_context_data(self, **kwargs):
        kwargs[self.context_object_name] = self.object
        return kwargs

    def get_success_url(self):
        return self.success_url

    def render_to_response(self, **kwargs):
        return render(
            self.request, 
            self.template_name, 
            context=self.get_context_data(**kwargs),
        )


class ArticleDeleteView(generic.DeleteView):
    model = Article
    pk_url_kwarg = 'id'
    success_url = reverse_lazy('index')
